package com.apress.proandroidmedia.ch06.httpaudioplaylistplayer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Vector;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import android.app.Activity;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnPreparedListener;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;

public class HTTPAudioPlaylistPlayer extends Activity implements
		OnClickListener, OnCompletionListener, OnPreparedListener {

	Vector playlistItems;

	Button parseButton;
	Button playButton;
	Button stopButton;

	EditText editTextUrl;
	String baseURL = "";
	MediaPlayer mediaPlayer;

	int currentPlaylistItemNumber = 0;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		parseButton = (Button) this.findViewById(R.id.ButtonParse);
		playButton = (Button) this.findViewById(R.id.PlayButton);
		stopButton = (Button) this.findViewById(R.id.StopButton);

		editTextUrl = (EditText) this.findViewById(R.id.EditTextURL);
		// editTextUrl.setText("http://live.kboo.fm:8000/high.m3u");
		editTextUrl.setText("http://pubint.ic.llnwd.net/stream/pubint_kmfa.m3u");
		
		parseButton.setOnClickListener(this);
		playButton.setOnClickListener(this);
		stopButton.setOnClickListener(this);

		playButton.setEnabled(false);
		stopButton.setEnabled(false);
		mediaPlayer = new MediaPlayer();
		mediaPlayer.setOnCompletionListener(this);
		mediaPlayer.setOnPreparedListener(this);
	}

	public void onClick(View view) {
		if (view == parseButton) {
			parsePlaylistFile();
		} else if (view == playButton) {
			playPlaylistItems();
		} else if (view == stopButton) {
			stop();
		}
	}

	private void parsePlaylistFile() {
		playlistItems = new Vector();

		HttpClient httpClient = new DefaultHttpClient();
		HttpGet getRequest = new HttpGet(editTextUrl.getText().toString());
		Log.v("URI", getRequest.getURI().toString());
		try {
			HttpResponse httpResponse = httpClient.execute(getRequest);
			if (httpResponse.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
				// ERROR MESSAGE
				Log.v("HTTP ERROR", httpResponse.getStatusLine()
						.getReasonPhrase());
			} else {
				InputStream inputStream = httpResponse.getEntity().getContent();
				BufferedReader bufferedReader = new BufferedReader(
						new InputStreamReader(inputStream));

				String line;
				while ((line = bufferedReader.readLine()) != null) {
					Log.v("PLAYLISTLINE", "ORIG: " + line);
					if (line.startsWith("#")) {
						// Metadata
						// Could do more with this but not for now
					} else if (line.length() > 0) {
						String filePath = "";
						if (line.startsWith("http://")) {
							// Assume it's a full URL
							filePath = line;
						} else {
							// Assume it's relative
							filePath = getRequest.getURI().resolve(line)
									.toString();
						}

						PlaylistFile playlistFile = new PlaylistFile(filePath);
						playlistItems.add(playlistFile);
					}
				}
				inputStream.close();
			}
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		playButton.setEnabled(true);
	}

	private void playPlaylistItems() {
		playButton.setEnabled(false);
		currentPlaylistItemNumber = 0;
		if (playlistItems.size() > 0) {
			String path = ((PlaylistFile) playlistItems
					.get(currentPlaylistItemNumber)).getFilePath();
			try {
				mediaPlayer.setDataSource(path);
				mediaPlayer.prepareAsync();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalStateException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void onPrepared(MediaPlayer _mediaPlayer) {
		stopButton.setEnabled(true);
		Log.v("HTTPAUDIOPLAYLIST", "Playing");
		mediaPlayer.start();
	}

	public void onCompletion(MediaPlayer mediaPlayer) {
		Log.v("ONCOMPLETE", "called");
		mediaPlayer.stop();
		mediaPlayer.reset();
		if (playlistItems.size() > currentPlaylistItemNumber + 1) {
			currentPlaylistItemNumber++;
			String path = ((PlaylistFile) playlistItems
					.get(currentPlaylistItemNumber)).getFilePath();
			try {
				mediaPlayer.setDataSource(path);
				mediaPlayer.prepareAsync();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalStateException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private void stop() {
		mediaPlayer.pause();
		playButton.setEnabled(true);
		stopButton.setEnabled(false);
	}

	class PlaylistFile {
		String filePath;

		public PlaylistFile(String _filePath) {
			filePath = _filePath;
		}

		public void setFilePath(String _filePath) {
			filePath = _filePath;
		}

		public String getFilePath() {
			return filePath;
		}
	}
}